;
;  tbeepr ver.1.1	copyright (c) 1995, 2013 introspec
;
;  beeper engine custom-made for the old game "tank battle". it was written under heavy
;  influence of the Wham! by Mark Alexander and the beeper engine by David Whittaker (as used
;  in Dizzy 1), so i am not fully sure how original some of these codes are. a lot of it
;  reads as completely new now. i decided to fix some obvious inefficiencies (because i can).
;

			DEVICE	ZXSPECTRUM48

			org	49152
	
EngineBegin:
TankBeeper:		ld	a, 0		; the number of track to play (0..5 for Tank Battle)
			di
			exx
			push	hl
			push	iy

			ld	l, a		; A = 7*A
			add	a
			add	l
			add	a
			add	l

			add	low TracksInfo	
			ld	l, a
			adc	high TracksInfo
			sub	l
			ld	h, a		; each track descriptor occupies 7 bytes, HL points at the correct one now
			
			ld	a, (hl)
			ld	(CurrentTempo+2), a
			inc	hl

			ld	(SaveSP+1), sp
			ld	sp, hl
			pop	hl
			ld	(PatternChannel1+1), hl
			pop	hl
			ld	(PatternChannel2+1), hl
			pop	hl
			ld	(PatternChannel3+1), hl
SaveSP:			ld	sp, 0		; pattern pointers have now been initialised

			call	PatternChannel1
			ld	(CurrentPosChannel1+1), hl
			call	PatternChannel2
			ld	(CurrentPosChannel2+1), hl
			call	PatternChannel3
			ld	(CurrentPosChannel3+1), hl

			ld	a, 7		; border colour
			ld	(BorderColour1+1), a
			ld	(BorderColour2+1), a
			ld	(BorderColour3+1), a

			ld	a, 1		; ensure that all channels are reloaded at the start
			ld	b, a
			exx
			ld	b, a
			ld	iyl, a
			jr	UpdateChannel1

;
;  this is the main ("fast") loop where the sound is generated. there are two channels, and every loop iteration updates one of the channels.
;  the switching between channels is achieved by swapping the register set (note the unbalanced exa : exx at the end of the loop).
;
;  the registers for each channel are as follows:
;  A - current port state
;  B - counter of "slow" loop iterations before changing the current note
;  D - counter of "fast" loop iterations before alternating the current output state
;  E - number of "fast" loop iterations during ON
;  H - 24 (when the sound is produced) or 0 (when the channel is silent)
;  L - number of "fast" loop iterations during OFF
;
;  IYL - counter of "slow" loop iterations before playing the next drum
;
;  IXH - "fast" loop iterations counter
;  IXL - "medium" loop iterations counter
;
;  "fast" loop repeats every 90 t-states. with two equivalent sound channels this corresponds to the discretization frequency
;  of 3500000/90/2 = 19444.4 hz. the "fast" loop always repeats 256 times, hence it lasts for 90*256 = 23040 t-states. "medium" loop
;  is used to define the tempo of the tune; the tempo is defined as the number of repetitions of the "fast" loop within the "medium"
;  loop. the tempo of 6 corresponds to the shortest note possible lasting 6*23040 = 138240t (~1/25th of a second).
;
;  i can see now that it is possible to speed up the main loop and squeeze more performance out of it. however, i decided to
;  leave the engine sound generator mostly intact and to only do small optimizations outside of the main sound loop, so that
;  the original track from the "tank battle" sounds without any changes.
;
;  channel one is also modified in a PWM-type fashion (with monotonously decreasing duty cycle). unfortunately, the speed
;  of this variation is independent of the frequency and/or tempo.
;
;  ok, i could not resist and modified the fast sound loop. it now runs at 3500000/88/2 = 19886.4 hz. the fact that the "fast" loop
;  has the number of tacts which is divisible by 8 is highly important on contended spectrums, where this improves the quality of
;  sound by quite a significant margin. 256 repetitions of the "fast" loop last for 88*256 = 22528 t-states.
;

PreCoreSoundLoop:	; the asymmetry of the "fast" loop reduces the impact of the "medium" loop (=better sound)
			nop : nop			; 8t	(padding the "fast" loop duration)
CoreSoundLoopExx:	exx				; 4t

CoreSoundLoop:		out	(254), a		; 11t
			dec	d			; 4t
			jr	z, AlternateOutput	; 12t/7t

CheckKeyboard:		ld	c, a			; 4t
			xor	a			; 4t
			in	a, (254)		; 11t
			cpl				; 4t
			and	31			; 7t
			ld	a, c			; 4t
			jr	z, CoreLoopPart2	; 10t => 11+4+7 + 4+4+11+4+7+4+12 = 68t

StopPlayback:		pop	iy
			pop	hl
			exx
			ei
			ret

AlternateOutput:	ret	nz			; 5t (waste tacts to align the duration of this branch)
			xor	h			; 4t
			bit	4, a			; 8t
			jp	nz, ItWasLow1		; 10t

ItWasHigh1:		ld	d, l			; 4t
			jp	CoreLoopPart2		; 10t

ItWasLow1: 		ld	d, e			; 4t
			jp	CoreLoopPart2		; 10t -> 11+4+12 + 5+4+8+10+4+10 = 68t

CoreLoopPart2:		dec	ixh			; 8t
			dec	ixh			; 8t
			ex	af, af'			; 4t -> 68t+8+8+4 = 88t

			out	(254), a		; 11t
			exx				; 4t
			dec	d			; 4t
			jr	nz, WasteTime		; 12t/7t

			xor	h			; 4t
			bit	4, a			; 8t
			jp	nz, ItWasLow2		; 10t
		
ItWasHigh2:		ld	d, l			; 4t
			jp	NextIteration		; 10t

ItWasLow2: 		ld	d, e			; 4t
			jp	NextIteration		; 10t -> 11+4+4+7 +4+8+10+4+10 = 62t
		
WasteTime:		jr	z, $+2			; 11+4+4+12 + X = 62t => X=31t
			db	24, 0, 24, 0

NextIteration:		ex	af, af'			; 4t
			jp	nz, PreCoreSoundLoop	; 10t -> 62t + 4+10 + 12t = 88t
		
			; this is the "medium" loop, which is responsible for tempo and envelope variations for channel 1
EnvelopeChannel2:	dec	e
FXType2:		jr	z, StepBackOverflow2
			inc	l
			db	#0E
StepBackOverflow2:	inc	e
			exx

			dec	e
FXType1:		jr	z, StepBackOverflow1	; could be JR Z, nn (duty cycle decay) or JR nn (constant duty cycle)
			inc	l			; adjust L so that the note period stays constant
			db	#0E			; LD C, n (a cheap way to skip the increment)
StepBackOverflow1:	inc	e


MediumLoopNext:		dec	ixl
			jp	nz, CoreSoundLoop


;
;  "slow" loop updates the current notes, plays drums and also executes "special" commands when necessary.
;  channels 1 and 2 can also have a choice of few crude duty cycle variations.
;

UpdateChannel1:		djnz	UpdateChannel2		; do we need to read the next note for channel 1?

ReadNoteChannel1:
CurrentPosChannel1:	ld	hl, 0			; current address within the current pattern
NextNoteChannel1:	ld	a, (hl)
			inc	hl
			ld	(CurrentPosChannel1+1), hl
			or	a
			jp	m, ProcessCommand1	; this is not a note, but a command that must be processed separately

			add	low FreqTable
			ld	l, a
			adc	high FreqTable
			sub	l
			ld	h, a
			ld	a, (hl)
			dec	a
DutyCycleChannel1:	jr	$+2

Ch1DutyCycle:
Ch1DutyCycle99:		ld	e, a			; 99% duty cycle for channel 1
			ld	l, 1
			jr	InitCounters1

Ch1DutyCycle01:		ld	l, a			; 1% duty cycle for channel 1
			ld	e, 1
			jr	InitCounters1

Ch1DutyCycle25:		srl	a			; 25% duty cycle for channel 1
Ch1DutyCycle50:		srl	a			; 50% duty cycle for channel 1
			jr	000f

Ch1DutyCycle75:		srl	a			; 75% duty cycle for channel 1
			ld	e, a
			srl	a
			add	e

000:			ld	e, a
			ld	a, (hl)
			sub	e
			ld	l, a

InitCounters1:		ld	h, 24			; H=24 for sound, H=0 for mute
PauseChannel1:		ld	d, 1			; D=1 will be updated during the first cycle iteration
BorderColour1:		ld	a, 0			; current output state is 0
DurationChannel1:	ld	b, 0

;
;  pretty much identical manipulations are done for channel 2. the old engine was a bit more limited in terms
;  of what it could do with the second channel, however, this update makes both channels fully equal.
;

UpdateChannel2:		ex	af, af'
			exx
			djnz	UpdateChannel3

CurrentPosChannel2:	ld	hl, 0
NextNoteChannel2:	ld	a, (hl)
			inc	hl
			ld	(CurrentPosChannel2+1), hl
			or	a
			jp	m, ProcessCommand2

			add	low FreqTable
			ld	l, a
			adc	high FreqTable
			sub	l
			ld	h, a
			ld	a, (hl)
			dec	a
DutyCycleChannel2:	jr	$+2		 ; 49384 24     6

Ch2DutyCycle:
Ch2DutyCycle99:		ld	e, a			; 99% duty cycle for channel 2
			ld	l, 1
			jr	InitCounters2

Ch2DutyCycle01:		ld	l, a			; 1% duty cycle for channel 2
			ld	e, 1
			jr	InitCounters2

Ch2DutyCycle25:		srl	a			; 25% duty cycle for channel 2
Ch2DutyCycle50:		srl	a			; 50% duty cycle for channel 2
			jr	000f

Ch2DutyCycle75:		srl	a			; 75% duty cycle for channel 2
			ld	e, a
			srl	a
			add	e

000:			ld	e, a
			ld	a, (hl)
			sub	e
			ld	l, a

InitCounters2:		ld	h, 24
PauseChannel2:		ld	d, 1
BorderColour2:		ld	a, 0
DurationChannel2:	ld	b, 0

;
;  third channel plays interrupting drums
;

UpdateChannel3:		ex	af, af'
CurrentTempo:		ld	ix, 6

			dec	iyl
			jp	nz, CoreSoundLoopExx

			push	af
			push	de
			push	hl
CurrentPosChannel3:	ld	hl, 0

NextDrumChannel3:	ld	a, (hl)
			inc	hl
			ld	(CurrentPosChannel3+1), hl
			or	a
			jp	m, ProcessCommand3

			ld	l, a
			add	a
			add	l
			add	low DrumsTable
			ld	l, a
			adc	high DrumsTable
			sub	l
			ld	h, a

			ld	a, ixl			; adjustement for the drum duration
			sub	(hl)
			ld	ixl, a

			push	bc
			inc	hl
			ld	c, (hl)
			inc	hl
			ld	b, (hl)
			call	DrumSynth
			pop	bc

SkipDrumming:		pop	hl
			pop	de
			pop	af
DurationChannel3:	ld	iyl, 0

			jp	CoreSoundLoopExx

;
;  the following section of the program is responsible for processing special codes occuring in channel 1 stream.
;  0-127 - offsets in the table of note periods (there are only 50 entries there, actually).
;  128 - mute channel
;  129 - stop playback and return from the player
;  130 - 1% duty cycle
;  131 - 25% duty cycle
;  132 - 50% duty cycle
;  133 - 75% duty cycle
;  134 - 99% duty cycle
;  135 - PWM envelope on (duty cycle decrease)
;  136 - PWM envelope off
;  137 - end of the pattern
;  223..255 - set new notes duration (in periods of the slow cycle)
;

ProcessCommand1:	sub	223
			jr	c, NotALength1
			ld	(DurationChannel1+1), a
			jp	NextNoteChannel1

NotALength1:		add	223-128
			add	a
			add	a
			ld	(CommandChannel1+1), a
CommandChannel1:	jr	$+2

Ch1_128:		ld	h, a
			jp	PauseChannel1

Ch1_129:		jp	StopPlayback
			nop

Ch1_130:		ld	a, low (Ch1DutyCycle01-Ch1DutyCycle)
			jr	UpdateDutyCycle1
Ch1_131:		ld	a, low (Ch1DutyCycle25-Ch1DutyCycle)
			jr	UpdateDutyCycle1
Ch1_132:		ld	a, low (Ch1DutyCycle50-Ch1DutyCycle)
			jr	UpdateDutyCycle1
Ch1_133:		ld	a, low (Ch1DutyCycle75-Ch1DutyCycle)
			jr	UpdateDutyCycle1
Ch1_134:		ld	a, low (Ch1DutyCycle99-Ch1DutyCycle)
			jr	UpdateDutyCycle1

Ch1_135:		ld	a, 40			; JR Z, nn (switch on PWM envelope)
			jr	UpdateFXType1
Ch1_136:		ld	a, 24			; JR nn (switch off PWM envelope)
			jr	UpdateFXType1

Ch1_137:		ld	hl, NextNoteChannel1
			push	hl

PatternChannel1:	ld	hl, 0
ReadNextPattern1:	ld	e, (hl)
			inc	hl
			ld	d, (hl)
			inc	hl
			ld	(PatternChannel1+1), hl
			ex	de, hl
			ld	a, h
			or	l
			ret	nz
			ex	de, hl
			ld	e, (hl)
			inc	hl
			ld	d, (hl)
			ex	de, hl
			jr	ReadNextPattern1

UpdateDutyCycle1:	ld	(DutyCycleChannel1+1), a
			jp	NextNoteChannel1

UpdateFXType1:		ld	(FXType1), a
			jp	NextNoteChannel1

;
;  the following section of the program is responsible for processing special codes occuring in channel 2 stream.
;  0-127 - offsets in the table of note periods (there are 50 entries there, actually).
;  128 - mute channel
;  129 - stop playback and return from the player
;  130 - 1% duty cycle
;  131 - 25% duty cycle
;  132 - 50% duty cycle
;  133 - 75% duty cycle
;  134 - 99% duty cycle
;  135 - PWM envelope on (duty cycle decrease)
;  136 - PWM envelope off
;  137 - end of the pattern
;  223..255 - set new notes duration (in periods of the slow cycle)
;

ProcessCommand2:	sub	223
			jr	c, NotALength2
			ld	(DurationChannel2+1), a
			jp	NextNoteChannel2

NotALength2:		add	223-128
			add	a
			add	a
			ld	(CommandChannel2+1), a
CommandChannel2:	jr	$+2

Ch2_128:		ld	h, a
			jp	PauseChannel2

Ch2_129:		jp	StopPlayback
			nop

Ch2_130:		ld	a, low (Ch2DutyCycle01-Ch2DutyCycle)
			jr	UpdateDutyCycle2
Ch2_131:		ld	a, low (Ch2DutyCycle25-Ch2DutyCycle)
			jr	UpdateDutyCycle2
Ch2_132:		ld	a, low (Ch2DutyCycle50-Ch2DutyCycle)
			jr	UpdateDutyCycle2
Ch2_133:		ld	a, low (Ch2DutyCycle75-Ch2DutyCycle)
			jr	UpdateDutyCycle2
Ch2_134:		ld	a, low (Ch2DutyCycle99-Ch2DutyCycle)
			jr	UpdateDutyCycle2

Ch2_135:		ld	a, 40			; JR Z, nn (switch on PWM envelope)
			jr	UpdateFXType2
Ch2_136:		ld	a, 24			; JR nn (switch off PWM envelope)
			jr	UpdateFXType2

Ch2_137:		ld	hl, NextNoteChannel2
			push	hl

PatternChannel2:        ld	hl, 0
ReadNextPattern2:	ld	e, (hl)
			inc	hl
			ld	d, (hl)
			inc	hl
			ld	(PatternChannel2+1), hl
			ex	de, hl
			ld	a, h
			or	l
			ret	nz
			ex	de, hl
			ld	e, (hl)
			inc	hl
			ld	d, (hl)
			ex	de, hl
			jr	ReadNextPattern2

UpdateDutyCycle2:	ld	(DutyCycleChannel2+1), a
			jp	NextNoteChannel2

UpdateFXType2:		ld	(FXType2), a
			jp	NextNoteChannel2

;
;  the following section of the program is responsible for processing special codes occuring in channel 3 stream.
;  0-127 - offsets in the table of drums (there are 20 entries there, actually).
;  128 - mute channel
;  129 - end of the pattern
;  223..255 - set new notes duration (in periods of the slow cycle)
;

ProcessCommand3:	sub	223
			jr	c, NotALength3
			ld	(DurationChannel3+2), a
			jp	NextDrumChannel3

NotALength3:		add	223-128
			add	a
			add	a
			ld	(CommandChannel3+1), a
CommandChannel3:	jr	$+2

Ch3_128:		jp	SkipDrumming
			nop

Ch3_129:		ld	hl, NextDrumChannel3
			push	hl

PatternChannel3:        ld	hl, 0
ReadNextPattern3:	ld	e, (hl)
			inc	hl
			ld	d, (hl)
			inc	hl
			ld	(PatternChannel3+1), hl
			ex	de, hl
			ld	a, h
			or	l
			ret	nz
			ex	de, hl
			ld	e, (hl)
			inc	hl
			ld	d, (hl)
			ex	de, hl
			jr	ReadNextPattern3

;
;  a universal routine to produce drum-like sounds (it is likely that i borrowed it from D.Whittaker's engine)
;
;  in order to stabilise the tempo properly, the parameters of the drum synth have been selected to closely
;  match the integer number of iterations of the "medium" loop. the drum noise lasts for 69*B-5+16*C*B+8*B^2 t-states.
;  the drums whose description starts with 1 are tuned to last for ~22528 t-states
;  the drums whose description starts with 2 are tuned to last for ~45056 t-states
;

DrumSynth:		ld	hl, EngineBegin+80
BorderColour3:		ld	d, 0

DrumSynthLoop:		ld	a, (hl)			; 7t
			inc	hl			; 6t
			and	24			; 7t
			or	d			; 4t
			out	(254), a		; 11t
			ld	a, c			; 4t
001:			dec	a			; 16t*C0-5
			jr	nz, 001b
			or	d			; 4t
			out	(254), a		; 11t
			ld	a, b			; 4t
002:			dec	a			; 16t*B-5
			jr	nz, 002b
			djnz	DrumSynthLoop		; 13t
			ret

DrumsTable:		; drums to replace 1 iteration of the medium loop
			db	1, 227, 6
			db	1, 193, 7
			db	1, 148, 9
			db	1, 107, 12
			db	1, 65, 18
			db	1, 37, 26
			db	1, 20, 34
			db	1, 17, 36
			db	1, 11, 40
			db	1, 7, 43
			; drums to replace 2 iterations of the medium loop
			db	2, 246, 11
			db	2, 206, 13
			db	2, 153, 17
			db	2, 101, 24
			db	2, 71, 31
			db	2, 44, 41
			db	2, 27, 50
			db	2, 18, 56
			db	2, 14, 59
			db	2, 9, 63

			DISPLAY	"engine code occupies: ", /D, $-EngineBegin, " bytes"

;
;  the new frequency table is properly recomputed from the standard note frequencies.
;  this is a table of relative errors achieved when using this table:
;  octave	c	c#	d	d#	e	f	f#	g	g#	a	a#	b
;  2					0.1	-0.11	0.086	0.004	0.037	0.25	0.13	0.25	-0.047
;  3		-0.018	-0.47	-0.47	0.2	0.46	0.17	-0.86	-0.9	0.5	-0.96	-0.88	1.4
;  4		-0.037	0.98	1.2	0.4	-1.8	0.34	1.7	2.1	1	-1.9	3.7	-3.3
;  5		-0.073	2	2.4	0.8	-3.6	-11.8	3.5	-11.5	2	15.4	-14.6	-6.6
;  6		-0.15	3.9	4.9	1.6	-7.3
;

FreqTable:		db	0, 241, 228, 215, 203, 192, 181, 171, 161		; great octave, starts at D#2 (biggest error 0.25%)
			db	152, 143, 135, 128, 121, 114, 107, 101, 96, 90, 85, 81	; small octave (biggest errors G3 -0.9%, A3 -1%, B3 1.4%)
			db	76, 72, 68, 64, 60, 57, 54, 51, 48, 45, 43, 40	; one-line octave (biggest errors G4 2.1%, A#4 3.7%, B4 -3.3%)
			db	38, 36, 34, 32, 30, 28, 27, 25, 24, 23, 21, 20	; two-line octave (biggest errors F5 -11.8%, A5 15.4%, A#5 -14.6%)
			db	19, 18, 17, 16, 15			; three-line octave, ends at E6 (biggest errors C#6 3.9%, D6 4.9%, E6 -7.3%)

			DISPLAY	"table of note periods occupies: ", /D, $-FreqTable, " bytes"

;
;  this is where the actual information about tunes is stored
;

TracksInfo:		db	6			; tempo
			dw	Track0Patterns1, Track0Patterns2, Track0Patterns3
			db	6
			dw	Track1Patterns1, Track1Patterns2, Track1Patterns3
			db	8
			dw	Track2Patterns1, Track2Patterns2, Track2Patterns3
			db	6
			dw	Track3Patterns1, Track3Patterns2, Track3Patterns3
			db	7
			dw	Track4Patterns1, Track4Patterns2, Track4Patterns3
			db	6
			dw	Track5Patterns1, Track5Patterns2, Track5Patterns3

; =============================================================================

			DEFINE	C1_MUTE	128
			DEFINE	C1_EXIT 129
			DEFINE	C1_DC01 130
			DEFINE	C1_DC25 131
			DEFINE	C1_DC50 132
			DEFINE	C1_DC75 133
			DEFINE	C1_DC99 134
			DEFINE	C1_ENV1 135
			DEFINE	C1_ENV0 136
			DEFINE	C1_PEND 137

			DEFINE	C2_MUTE	128
			DEFINE	C2_EXIT	129
			DEFINE	C2_DC01 130
			DEFINE	C2_DC25 131
			DEFINE	C2_DC50 132
			DEFINE	C2_DC75 133
			DEFINE	C2_DC99 134
			DEFINE	C2_ENV1 135
			DEFINE	C2_ENV0 136
			DEFINE	C2_PEND 137

			DEFINE	C3_MUTE 128
			DEFINE	C3_PEND	129

			DEFINE	C3_DRM0 0
			DEFINE	C3_DRM1 1
			DEFINE	C3_DRM2 2
			DEFINE	C3_DRM3 3
			DEFINE	C3_DRM4 4
			DEFINE	C3_DRM5 5
			DEFINE	C3_DRM6 6
			DEFINE	C3_DRM7 7
			DEFINE	C3_DRM8 8
			DEFINE	C3_DRM9 9

; =============================================================================

Track0Patterns1:	dw	Track0Channel1
Track0Patterns2:	dw	Track0Channel2
Track0Patterns3:	dw	Track0Channel3
			dw	0, Track0Patterns3

Track0Channel1:		db	C1_ENV1, C1_DC50, 227, 26, C1_MUTE, 26, 24, 26, 24, 26
			db	26, C1_DC99, 231, 29, 26, 24, 26, C1_DC50, 227, 26
			db	24, 26, 24, 26, 24, 26, 26, C1_DC99, 231, 29
			db	26, 24, 24, C1_DC50, 227, 31, 29, 31, 29, 31
			db	29, 31, 31, C1_DC99, 231, 34, 31, 29, 31, C1_DC50

			db	227, 31, 29, 31, 29, 31, 29, 31, 31, 231
			db	34, 31, 239, 29, C1_DC99, 227, 28, 26, 28, 26
			db	28, 26, 28, 28, 231, 31, 28, 26, 231, 28
			db	227, 28, 26, 28, 26, 28, 26, 28, C1_MUTE, 231
			db	35, 33, 31, 33, 227, 31, 29, 31, 29, 27

			db	29, 27, 29, 231, 24, 27, 31, 36, 227, 27
			db	26, 27, 26, 27, 26, 27, C1_MUTE, 231, 24, 27
			db	31, 36, 227, C1_MUTE, 32, C1_MUTE, 31, 32, 34, 32
			db	31, C1_MUTE, 32, C1_MUTE, 31, 32, 31, 32, 34, 231
			db	36, 32, 27, 227, 32, 29, 231, 24, 26, 27

			db	26, 31, 32, 27, 29, 31, 32, 29, 33, 26
			db	227, C1_MUTE, 26, 29, 26, 31, 29, 231, 31, 227
			db	C1_MUTE, 31, 231, 35, 227, 34, C1_ENV0, C1_DC50, 235, 33, C1_ENV1
			db	227, 31, 33, 36, 33, 31, C1_ENV0, C1_DC50, 235, 33, C1_ENV1
			db	227, 31, 33, 36, 33, 31, C1_ENV0, C1_DC50, 235, 31, C1_ENV1

			db	227, 31, 33, 36, 33, 31, C1_ENV0, 231, 31, C1_ENV1
			db	227, 33, 31, 33, 36, 33, 31, C1_ENV0, C1_DC50, 235, 33
			db	C1_ENV1, 227, 31, 33, 36, 33, 31, C1_ENV0, C1_DC50, 235, 33
			db	C1_ENV1, 227, 31, 33, 36, 33, 31, C1_ENV0, C1_DC50, 235, 31
			db	C1_ENV1, 227, 31, 33, 36, 33, 31, C1_ENV0, C1_DC50, 235, 31

			db	C1_ENV1, 227, 31, C1_ENV0, C1_DC50, 235, 33, C1_ENV1, 231, 31, C1_EXIT

Track0Channel2:		db	C2_ENV0, C2_DC50, 227, 2, C2_MUTE, 2, C2_MUTE, 5, C2_MUTE, 231, 2
			db	227, C2_MUTE, 231, 2, 227, C2_MUTE, 5, C2_MUTE, 2, C2_MUTE
			db	2, C2_MUTE, 2, C2_MUTE, 5, C2_MUTE, 231, 2, 227, C2_MUTE
			db	231, 2, 227, C2_MUTE, 5, C2_MUTE, 2, C2_MUTE, 7, C2_MUTE
			db	7, C2_MUTE, 10, C2_MUTE, 231, 7, 227, C2_MUTE, 231, 7

			db	227, C2_MUTE, 10, C2_MUTE, 7, C2_MUTE, 7, C2_MUTE, 7, C2_MUTE
			db	10, C2_MUTE, 231, 7, 227, C2_MUTE, 231, 7, 227, C2_MUTE
			db	10, C2_MUTE, 7, C2_MUTE, 9, C2_MUTE, 9, C2_MUTE, 12, C2_MUTE
			db	231, 9, 227, C2_MUTE, 231, 9, 227, C2_MUTE, 12, C2_MUTE
			db	9, C2_MUTE, 9, C2_MUTE, 9, C2_MUTE, 12, C2_MUTE, 231, 9

			db	227, C2_MUTE, 231, 9, 227, C2_MUTE, 12, C2_MUTE, 9, C2_MUTE
			db	0, C2_MUTE, 0, C2_MUTE, 3, C2_MUTE, 231, 0, 227, C2_MUTE
			db	231, 0, 227, C2_MUTE, 231, 3, 227, 0, C2_MUTE, 0
			db	C2_MUTE, 0, C2_MUTE, 3, C2_MUTE, 231, 0, 227, C2_MUTE, 231
			db	0, 227, C2_MUTE, 231, 3, 227, 0, C2_MUTE, 5, C2_MUTE

			db	5, C2_MUTE, 3, C2_MUTE, 5, C2_MUTE, 5, C2_MUTE, 5, C2_MUTE
			db	3, C2_MUTE, 5, C2_MUTE, 5, C2_MUTE, 5, C2_MUTE, 3, C2_MUTE
			db	5, C2_MUTE, 6, C2_MUTE, 5, C2_MUTE, 3, C2_MUTE, 5, C2_MUTE
			db	7, C2_MUTE, 7, C2_MUTE, 5, C2_MUTE, 7, C2_MUTE, 231, 11
			db	227, 12, C2_MUTE, 10, C2_MUTE, 9, C2_MUTE, 7, C2_MUTE, 7

			db	C2_MUTE, 5, C2_MUTE, 7, C2_MUTE, 12, C2_MUTE, 12, C2_MUTE, 231
			db	11, 227, 10, 231, 9, 227, C2_MUTE, 9, C2_MUTE, 12
			db	C2_MUTE, 231, 9, 227, C2_MUTE, 231, 9, 227, C2_MUTE, 12
			db	C2_MUTE, 9, C2_MUTE, 7, C2_MUTE, 7, C2_MUTE, 10, C2_MUTE, 231
			db	7, 227, C2_MUTE, 231, 7, 227, C2_MUTE, 10, C2_MUTE, 7

			db	C2_MUTE, 9, C2_MUTE, 9, C2_MUTE, 12, C2_MUTE, 231, 9, 227
			db	C2_MUTE, 231, 9, 227, C2_MUTE, 12, C2_MUTE, 9, C2_MUTE, 7
			db	C2_MUTE, 7, C2_MUTE, 10, C2_MUTE, 231, 7, 227, C2_MUTE, 231
			db	7, 227, C2_MUTE, 10, C2_MUTE, 7, C2_MUTE, C2_PEND

Track0Channel3:		db	227, C3_DRM9, C3_DRM0+10, C3_DRM9, C3_MUTE, C3_DRM2+10, C3_DRM0+10, C3_MUTE, C3_DRM5, C3_DRM9
			db	C3_DRM0+10, C3_DRM9, C3_MUTE, C3_DRM2+10, C3_MUTE, C3_DRM6, C3_MUTE, C3_PEND

; =============================================================================

Track1Patterns1:	dw	Track1Channel1
Track1Patterns2:	dw	Track1Channel2
Track1Patterns3:	dw	Track1Channel3
			dw	0, Track1Patterns3

Track1Channel1:		db	C1_ENV1, C1_DC99, 231, 7, 227, 7, 235, 12, 231, 12
			db	227, 12, 235, 14, 231, 14, 227, 14, 19, C1_EXIT

Track1Channel2:		db	C2_DC01, 227, 7, C2_MUTE, 7, 16, 231, C2_MUTE, 227, 12
			db	C2_MUTE, 12, 7, 231, C2_MUTE, 227, 7, C2_MUTE, 7, 12
			db	C2_PEND

Track1Channel3:		db	255, C3_MUTE, C3_MUTE, C3_MUTE, C3_PEND

; =============================================================================

Track2Patterns1:	dw	Track2Channel1
Track2Patterns2:	dw	Track2Channel2
Track2Patterns3:	dw	Track2Channel3
			dw	0, Track2Patterns3

Track2Channel1:		db	C1_ENV1, C1_DC99, 231, 24, 227, 26, 235, 27, 231, 26
			db	227, 24, 235, 24, 231, 22, 227, 18, 235, 21
			db	239, 19, 227, C1_MUTE, C1_EXIT

Track2Channel2:		db	C2_DC50, 227, 12, C2_MUTE, 14, 15, 231, C2_MUTE, 227, 14
			db	C2_MUTE, 12, 12, 231, C2_MUTE, 227, 10, C2_MUTE, 6, 9
			db	231, C2_MUTE, 239, 7, 227, C2_MUTE, C2_PEND

Track2Channel3:		db	227, C3_DRM7, C3_MUTE, C3_DRM7, 235, C3_DRM7, C3_PEND

; =============================================================================

Track3Patterns1:	dw	Track3Channel1
Track3Patterns2:	dw	Track3Channel2
Track3Patterns3:	dw	Track3Channel3
			dw	0, Track3Patterns3

Track3Channel1:		db	C1_ENV1, C1_DC25, 227, 2, 2, 2, 2, 2, 2, 2
			db	2, 2, 2, 2, 2, 2, 2, 2, 2, 9
			db	9, 9, 9, 9, 9, 9, 9, 2, 2, 2
			db	2, 9, 9, 9, 9, 2, 2, 9, 9, 2
			db	2, 2, C1_EXIT

Track3Channel2:		db	C2_ENV0, C2_DC75, 239, 14, 231, 18, 21, 255, 26, 28, 239
			db	30, 28, 231, 26, 21, 235, 26, C2_PEND

Track3Channel3:		db	255, C3_DRM8, 239, C3_DRM8, C3_DRM8, 231, C3_DRM8, C3_DRM8, C3_DRM8, C3_DRM8
			db	227, C3_DRM8, C3_DRM8, C3_DRM8, C3_DRM5, C3_DRM8, C3_DRM8, C3_DRM8, C3_DRM5, C3_DRM8
			db	C3_MUTE, C3_DRM8, C3_MUTE, C3_DRM8, 235, C3_MUTE, C3_PEND

; =============================================================================

Track4Patterns1:	dw	Track4Channel1
			dw	0, Track4Patterns1
Track4Patterns2:	dw	Track4Channel2
			dw	0, Track4Patterns2
Track4Patterns3:	dw	Track4Channel3
			dw	0, Track4Patterns3

Track4Channel1:		db	C1_ENV1, C1_DC50, 239, 31, 227, 29, 27, 235, 26, 231
			db	22, 227, 26, 239, 30, 227, 31, 33, 247, 31
			db	C1_PEND

Track4Channel2:		db	C2_DC01, 239, 27, 227, 26, 24, 235, 22, 231, 19
			db	227, 22, 235, 26, 227, 24, 22, 21, 247, 19
			db	C2_PEND

Track4Channel3:		db	235, C3_DRM0, C3_PEND

; =============================================================================

Track5Patterns1:	dw	Track5Channel1
Track5Patterns2:	dw	Track5Channel2
Track5Patterns3:	dw	Track5Channel3
			dw	0, Track5Patterns3

Track5Channel1:		db	C1_ENV1, C1_DC99, 226, 21, 232, C1_MUTE, 226, 21, 24, 23
			db	21, 28, 232, C1_MUTE, 226, 16, 20, 18, 16, 21
			db	232, C1_MUTE, 226, 21, 24, 23, 21, 232, 28, 226
			db	C1_MUTE, 232, 16, 226, C1_MUTE, 21, 232, C1_MUTE, 226, 21
			db	24, 23, 21, 28, 232, C1_MUTE, 226, 16, 20, 18

			db	16, 229, 21, C1_MUTE, 226, 21, 24, 23, 21, 232
			db	28, 226, C1_MUTE, 232, 16, 226, C1_MUTE, 229, 33, 226
			db	C1_MUTE, 229, 33, 226, 34, 33, 31, 229, 29, C1_MUTE
			db	226, 26, 29, 28, 26, 229, 31, C1_MUTE, 226, 31
			db	33, 31, 29, 229, 28, C1_MUTE, 226, 24, 28, 26

			db	24, 229, 29, C1_MUTE, 226, 23, 26, 24, 23, 235
			db	28, 226, 21, 24, 23, 21, 229, 17, 26, 16
			db	28, 232, 21, 226, C1_MUTE, 232, 33, 226, C1_MUTE, 33
			db	232, C1_MUTE, 226, 33, 34, 33, 31, 229, 29, C1_MUTE
			db	226, 26, 29, 28, 26, 229, 31, C1_MUTE, 226, 31

			db	33, 31, 29, 229, 28, C1_MUTE, 226, 24, 28, 26
			db	24, 229, 29, C1_MUTE, 226, 23, 26, 24, 23, 229
			db	28, C1_MUTE, 226, 21, 24, 23, 21, 229, 17, 26
			db	16, 28, 232, 21, 226, C1_MUTE, 232, 21, 226, C1_MUTE
			db	229, 21, 24, 28, 235, 33, 229, 36, 33, 28

			db	244, 33, 226, C1_MUTE, C1_EXIT

Track5Channel2:		db	C2_ENV1, C2_DC75, 247, 9, 4, 9, 4, 9, 4, 9, 4
			db	9, 2, 7, 0, 235, 5, 11, 247, 9, 235
			db	2, 4, 247, 9, 9, 2, 7, 0, 235, 5
			db	11, 247, 9, 235, 2, 4, 247, 9, 229, 9
			db	12, 16, 12, 9, 12, 16, 12, 16, 241, 21
			db	C2_PEND

Track5Channel3:		db	224, C3_DRM9, C3_DRM2, C3_MUTE, 226, C3_MUTE, C3_MUTE, C3_DRM4, C3_DRM1, C3_MUTE
			db	C3_DRM6, C3_MUTE, C3_DRM9, C3_MUTE, C3_DRM5, C3_MUTE, C3_DRM2, C3_DRM1, C3_DRM0, C3_MUTE
			db	C3_PEND

; =============================================================================

			DISPLAY	"tank battle tracks occupy: ", /D, $-TracksInfo, " bytes"

			IF _ERRORS = 0

			;SAVESNA	"tbeepr.sna", 49152

			SAVEBIN	"tbeepr.bin", EngineBegin, $-EngineBegin

			ELSE

			SHELLEXEC "cmd /c del tbeepr.bin tb_ost.tap"

			ENDIF
